local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

local IsItemInRange = IsItemInRange
local math = math
local pairs = pairs

c.Init(a)

------------------------------------------------------------------------ common
local itemRanges = {
--	[32321] = 10, -- Sparrowhawk Net
	[46722] = 15, -- Grol'dom Net
	[10645] = 20, -- Gnomish Death Ray
	[24268] = 25, -- Netherweave Net
	[835] = 30, -- Large Rope Net
	[41509] = 35, -- Frostweave Net
--	[28767] = 40, -- The Decapitator
--	[32698] = 45, -- Wrangling Rope
--	[35278] = 80, -- Reinforced Net
}

-- pre-fetch whatever wow needs to cache for the items
for id, _ in pairs(itemRanges) do
	IsItemInRange(id)
end

local function getMinDistance()
	local target = s.UnitSelection()
	if not s.Enemy(target) then
		return 1000
	end
	
	local min = 0
	for itemId, range in pairs(itemRanges) do
		if range > min and IsItemInRange(itemId, target) == 0 then
			min = range
		end
	end
	return min
end

--local function needsInner()
--	return not s.Buff(c.GetID("Inner Fire"), "player")
--		and not s.Buff(c.GetID("Inner Will"), "player")
--end

c.AddOptionalSpell("Power Word: Fortitude", nil, {
	NoRangeCheck = true,
	CheckFirst = function()
		return c.RaidBuffNeeded(c.STAMINA_BUFFS)
	end
})

c.AddOptionalSpell("Inner Fire", nil, {
	Buff = "Inner Fire",
	BuffUnit = "player",
})

c.AddOptionalSpell("Power Infusion", nil, {
	NoGCD = true,
	NoRangeCheck = true,
})
--
--c.AddOptionalSpell("Flash Heal", "under Surge of Light", {
--	NoRangeCheck = true,
--	NoStopChannel = true,
--	CheckFirst = function()
--		return c.HasBuff("Surge of Light")
--	end
--})
--
--c.AddOptionalSpell("Binding Heal", nil, {
--	NoRangeCheck = true,
--	CheckFirst = function()
--		local target = nil
--		if a.GetConfig("mouseover") then
--			target = "mouseover"
--		else
--			target = s.UnitSelection()
--		end
--		return not UnitIsUnit(target, "player")
--			and UnitIsFriend(target, "player")
--			and s.HealthDamage(target) > 30000
--			and s.HealthDamage("player") > 30000
--	end
--})
--
--c.AddOptionalSpell("Shadowfiend", "for Mana", {
--	NoStopChannel = true,
--	CheckFirst = function()
--		return s.PowerPercent("player") < 70
--	end
--})
--
--c.AddOptionalSpell("Inner Fire", "unless Inner Will", {
--	CheckFirst = needsInner,
--})
--
--c.AddOptionalSpell("Inner Will", nil, {
--	Buff = "Inner Will",
--	BuffUnit = "player",
--})
--
--c.AddOptionalSpell("Inner Will", "unless Inner Fire", {
--	CheckFirst = needsInner,
--})

c.AddOptionalSpell("Desperate Prayer", nil, {
	Override = function()
		return s.HealthPercent("player") < 70
			and s.HasSpell(c.GetID("Desperate Prayer"))
			and c.GetCooldown("Desperate Prayer") == 0
	end
})

c.AddOptionalSpell("Halo", nil, {
	Override = function(z)
		if not s.HasSpell(c.GetID("Halo")) or c.GetCooldown("Halo") > 0 then
			return false
		end
		
		z.FlashSize = nil
		local dist = getMinDistance()
		if dist >= 25 then
			z.FlashColor = "red"
		elseif dist >= 20 then
			if not CheckInteractDistance(s.UnitSelection(), 1) then
				z.FlashSize = s.FlashSizePercent() / 2
			end
			z.FlashColor = "green"
		elseif dist >= 15 then
			z.FlashColor = "yellow"
		else
			z.FlashColor = "red"
		end
		return true
	end
})

c.AddSpell("Cascade", nil, {
	Override = function(z)
		if not s.HasSpell(c.GetID("Cascade")) 
			or c.GetCooldown("Cascade") > 0 then
			
			return false
		end
		
		local dist = getMinDistance()
		if dist >= 40 then
			return false
		elseif dist >= 30 then
			z.FlashColor = "green"
		elseif dist >= 20 then
			z.FlashColor = "yellow"
		else
			z.FlashColor = "red"
		end
		return true
	end
})

------------------------------------------------------------------------ shadow
c.AddOptionalSpell("Shadowform", nil, {
	Type = "form",
})

c.AddOptionalSpell("Shadowfiend", nil, {
	Override = function()
		return c.GetCooldown("Shadowfiend") == 0
	end
})

c.AddOptionalSpell("Mindbender", nil, {
	Override = function()
		return c.GetCooldown("Mindbender") == 0
	end
})

c.AddSpell("Devouring Plague", nil, {
	EarlyRefresh = 99,
	Override = function(z)
		return a.Orbs >= 3 
			and c.GetMyDebuffDuration("Devouring Plague") < z.EarlyRefresh
	end
})
c.ManageDotRefresh("Devouring Plague", 3)

c.AddSpell("Devouring Plague", "to Prevent Cap", {
	Override = function(z)
		return a.Orbs >= 3
			and (c.GetCooldown("Mind Blast") < 1.5
				or (s.HealthPercent() < 20 
					and c.GetCooldown("Shadow Word: Death") < 1.5
					and a.SWDinARow == 0))
	end
})

c.AddSpell("Shadow Word: Death", nil, {
	Override = function(z)
		return s.HealthPercent() < 20
			and (a.SWDinARow == 1
				or (a.SWDinARow == 0 
					and c.GetCooldown("Shadow Word: Death") == 0))
	end
})

c.AddSpell("Shadow Word: Death", "wait", {
	FlashColor = "green",
	FlashSize = s.FlashSizePercent() / 2,
	Override = function(z)
		return a.SWDinARow < 2
			and s.HealthPercent() < 20
			and c.GetCooldown("Shadow Word: Death") < .5
	end
})

c.AddSpell("Mind Blast", nil, {
	Override = function()
		return c.GetCooldown("Mind Blast") == 0
			and not c.IsCasting("Mind Blast")
	end
})

c.AddSpell("Mind Blast", "wait", {
	FlashColor = "green",
	FlashSize = s.FlashSizePercent() / 2,
	Override = function()
		return c.GetCooldown("Mind Blast") < .5
			and not c.IsCasting("Mind Blast")
	end
})

c.AddSpell("Shadow Word: Pain", nil, {
	EarlyRefresh = 99,
	Override = function(z)
		return c.GetMyDebuffDuration("Shadow Word: Pain") < z.EarlyRefresh
			and not c.IsCastingOrInAir("Shadow Word: Pain")
	end
})
c.ManageDotRefresh("Shadow Word: Pain", 3)

c.AddSpell("Shadow Word: Pain", "application", {
	Override = function(z)
		return not c.HasMyDebuff("Shadow Word: Pain")
			and not c.IsCastingOrInAir("Shadow Word: Pain")
	end
})

c.AddSpell("Vampiric Touch", nil, {
	EarlyRefresh = 99,
	Override = function(z)
		return c.GetMyDebuffDuration("Vampiric Touch") 
				< c.GetCastTime("Vampiric Touch") + z.EarlyRefresh
			and not c.IsCastingOrInAir("Vampiric Touch")
	end
})
c.ManageDotRefresh("Vampiric Touch", 3)

c.AddSpell("Vampiric Touch", "application", {
	Override = function(z)
		return c.GetMyDebuffDuration("Vampiric Touch") 
				< c.GetCastTime("Vampiric Touch")
			and not c.IsCastingOrInAir("Vampiric Touch")
	end
})

c.AddSpell("Shadow Word: Insanity", nil, {
	Override = function()
		local duration = c.GetMyDebuffDuration("Shadow Word: Pain")
		return duration > 0 
			and duration < c.GetSpell("Shadow Word: Pain").EarlyRefresh * 2
			and not c.IsCasting("Shadow Word: Insanity")
	end
})

c.AddSpell("Mind Spike", "under Surge of Darkness", {
	Override = function()
		return a.Surges > 0
	end
})

c.AddSpell("Mind Spike", "under Surge of Darkness Cap", {
	Override = function()
		return a.Surges == 2
	end
})

c.AddOptionalSpell("Dispursion", nil, {
	CheckFirst = function()
		return s.PowerPercent("player") < 64
	end
})
